/*
    ClipGrab³
    Copyright (C) Philipp Schmieder
    http://clipgrab.de
    feedback [at] clipgrab [dot] de

    This file is part of ClipGrab.
    ClipGrab is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    ClipGrab is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ClipGrab.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "converter_ffmpeg.h"

void ffmpegThread::run()
{
    QString videoCodec;
    QString videoBitrate;
    QString audioCodec;

    bool audioCodecAccepted = false;
    bool videoCodecAccepted = false;

    #ifdef Q_WS_MAC
        ffmpegCall =  "\"" + QApplication::applicationDirPath() + "/ffmpeg\" -i \"" + inputFile->fileName() + "\" -y";
    #else
        ffmpegCall = "ffmpeg -i \"" + inputFile->fileName() + "\" -y";
    #endif


    QObject* parent = new QObject;

    ffmpeg = new QProcess(parent);
    ffmpeg->start(ffmpegCall + " -codecs");
    ffmpeg->waitForFinished(-1);
    QString videoInfo = ffmpeg->readAllStandardError();
    ffmpeg->close();

    QRegExp expression;
    expression = QRegExp("Audio: (.*),");
    expression.setMinimal(true);
    if (expression.indexIn(videoInfo) !=-1)
    {
        audioCodec = expression.cap(1);
    }
    expression = QRegExp("Video: (.*),");
    expression.setMinimal(true);
    if (expression.indexIn(videoInfo) !=-1)
    {
        videoCodec = expression.cap(1);
    }

    expression = QRegExp("Video:.*([0-9]+) kb/s");
    expression.setMinimal(true);
    if (expression.indexIn(videoInfo) !=-1)
    {
        videoBitrate = expression.cap(1);
    }

    qDebug() << "Source video: " << videoCodec << videoBitrate << audioCodec;
    qDebug() << "Target video: " << acceptedVideoCodec << acceptedAudioCodec;

    for (int i = 0; i < acceptedAudioCodec.size(); ++i)
    {
        if (acceptedAudioCodec[i] == audioCodec)
        {
            audioCodecAccepted = true;
        }
    }

    for (int i = 0; i < acceptedVideoCodec.size(); ++i)
    {
        if (acceptedVideoCodec[i] == videoCodec)
        {
            videoCodecAccepted = true;
        }
    }

    if (acceptedAudioCodec[0] == "none")
    {
        ffmpegCall = ffmpegCall + " -an";
    }
    else
    {
        if (audioCodecAccepted == true)
        {
            ffmpegCall = ffmpegCall + " -acodec copy";
        }
        else
        {
            if (acceptedAudioCodec[0] == "libvorbis")
            {
                if (videoInfo.contains("libvorbis"))
                {
                    ffmpegCall = ffmpegCall + " -acodec libvorbis -aq 9";
                }
                else
                {
                    ffmpegCall = ffmpegCall + " -acodec vorbis -aq 9 -strict experimental";
                }
            }
            else
            {
                ffmpegCall = ffmpegCall + " -acodec " + acceptedAudioCodec[0] + " -ab 256k";
            }
        }
    }

    if (acceptedVideoCodec[0] == "none")
    {
        ffmpegCall = ffmpegCall + " -vn";
    }
    else
    {
        if (videoCodecAccepted == true)
        {
            ffmpegCall = ffmpegCall + " -vcodec copy";
        }
        else
        {
            if (videoBitrate.toInt() > 100)
            {
                ffmpegCall = ffmpegCall + " -vb " + QString::number(videoBitrate.toInt()*1.2) + "k" + " -vcodec " + acceptedVideoCodec[0];
            }
            else
            {
                ffmpegCall = ffmpegCall + " -sameq -vcodec " + acceptedVideoCodec[0];
            }
        }
    }

    ffmpegCall = ffmpegCall + " -metadata title=\"" + metaTitle + "\"";
    ffmpegCall = ffmpegCall + " -metadata author=\"" + metaArtist + "\"";
    ffmpegCall = ffmpegCall + " -metadata artist=\"" + metaArtist + "\"";

    ffmpegCall = ffmpegCall + " \"" + target + "\"";
    qDebug() << "Executing ffmpeg: " << ffmpegCall;

    ffmpeg = new QProcess(parent);
    ffmpeg->start(ffmpegCall);
    ffmpeg->waitForFinished(-1);
    qDebug() << ffmpeg->readAllStandardError();
    ffmpeg->close();

    // patch added for Qt Version 4.5.0 by Günther Bauer
    #if QT_VERSION == 0x040500
        QFile::remove(randomFile);
    #endif
    // end of patch
};

converter_ffmpeg::converter_ffmpeg()
{
        this->_modes.append(tr("MPEG4"));
        this->_modes.append(tr("WMV (Windows)"));
        this->_modes.append(tr("OGG Theora"));
        this->_modes.append(tr("MP3 (audio only)"));
        this->_modes.append(tr("OGG Vorbis (audio only)"));
}


QString converter_ffmpeg::getExtensionForMode(int mode)
{
    switch (mode)
    {
        case 0:
            return "mp4";
            break;
        case 1:
            return "wmv";
            break;
        case 2:
            return "ogg";
            break;
        case 3:
            return "mp3";
            break;
        case 4:
            return "ogg";
            break;
    }
    return "";
}



void converter_ffmpeg::startConversion(QFile* inputFile, QString& target, QString /*originalExtension*/, QString metaTitle, QString metaArtist, int mode)
{
    QStringList acceptedAudio;
    QStringList acceptedVideo;
    QString container;
    QDir fileCheck;

    switch (mode)
    {
    case 0:
        acceptedAudio <<  "libvo_aacenc" << "aac" << "mp3";
        acceptedVideo << "mpeg4" << "h264";
        container = "mp4";
        break;
    case 1:
        acceptedAudio << "wmav2";
        acceptedVideo << "wmv2";
        container = "wmv";
        break;
    case 2:
        acceptedAudio << "libvorbis" << "vorbis";
        acceptedVideo << "libtheora" << "theora";
        container = "ogv";
        break;
    case 3:
        acceptedAudio << "libmp3lame" << "mp3";
        acceptedVideo << "none";
        container = "mp3";
        break;
    case 4:
        acceptedAudio << "libvorbis" << "vorbis";
        acceptedVideo << "none";
        container = "ogg";
        break;
    }


    if (fileCheck.exists(target + "." + container))
    {
        int i = 1;
        while (fileCheck.exists(target + "-" + QString::number(i) + container))
        {
            i++;
        }
        target.append("-");
        target.append(QString::number(i));
    }

    target.append("." + container);

    ffmpeg.inputFile = inputFile;
    ffmpeg.acceptedAudioCodec = acceptedAudio;
    ffmpeg.acceptedVideoCodec = acceptedVideo;
    ffmpeg.metaTitle = metaTitle;
    ffmpeg.metaArtist = metaArtist;
    ffmpeg.target = target;
    connect(&ffmpeg, SIGNAL(finished()), this, SLOT(emitFinished()));
    ffmpeg.start();

}


converter* converter_ffmpeg::createNewInstance()
{
    return new converter_ffmpeg();
}

void converter_ffmpeg::emitFinished()
{
    ffmpeg.inputFile->deleteLater();
    emit conversionFinished();
}
